@@ -5,8 +5,15 @@
 01. Read BIDS datasets
 ======================
 
-When working with electrophysiological data in the BIDS format, we usually have
-varying data types, which can be loaded via the ``read_raw_bids`` function.
+When working with electrophysiological data in the BIDS format, an important
+resource is the `OpenNeuro <https://openneuro.org/>`_ database. OpenNeuro
+works great with MNE-BIDS because every dataset must pass a validator
+that tests to ensure its format meets BIDS specifications before the dataset
+can be uploaded, so you know the data will work with a script like in this
+example without modification.
+
+We have various data types that can be loaded via the ``read_raw_bids``
+function:
 
 - MEG
 - EEG (scalp electrodes)
@@ -19,99 +26,116 @@ inspect BIDS-formatted data.
 """
 # Authors: Adam Li <adam2392@gmail.com>
 #          Richard Höchenberger <richard.hoechenberger@gmail.com>
+#          Alex Rockhill <aprockhill@mailbox.org>
 #
 # License: BSD (3-clause)
 
-###############################################################################
+# %%
 # Imports
 # -------
 # We are importing everything we need for this example:
-from mne.datasets import somato
+import os
+import os.path as op
+import openneuro
 
+from mne.datasets import sample
 from mne_bids import BIDSPath, read_raw_bids, print_dir_tree, make_report
 
-###############################################################################
-# We will be using the `MNE somato data <mne_somato_data_>`_, which
-# is already stored in BIDS format.
-# For more information, you can check out the
-# respective :ref:`example <ex-convert-mne-sample>`.
-
-###############################################################################
-# Download the ``somato`` BIDS dataset
-# ------------------------------------
+# %%
+# Download a subject's data from an OpenNeuro BIDS dataset
+# --------------------------------------------------------
+#
+# Download the data, storing each in a ``target_dir`` target directory, which,
+# in ``mne-bids`` terminology, is the `root` of each BIDS dataset. This example
+# uses this `EEG dataset <https://openneuro.org/datasets/ds002778>`_ of
+# resting-state recordings of patients with Parkinson's disease.
 #
-# Download the data if it hasn't been downloaded already, and return the path
-# to the download directory. This directory is the so-called `root` of this
-# BIDS dataset.
-bids_root = somato.data_path()
 
-###############################################################################
+# .. note: If the keyword argument include is left out of
+#          ``openneuro.download``, the whole dataset will be downloaded.
+#          We're just using data from one subject to reduce the time
+#          it takes to run the example.
+
+dataset = 'ds002778'
+subject = 'pd6'
+
+# Download one subject's data from each dataset
+bids_root = op.join(op.dirname(sample.data_path()), dataset)
+if not op.isdir(bids_root):
+    os.makedirs(bids_root)
+
+openneuro.download(dataset=dataset, target_dir=bids_root,
+                   include=[f'sub-{subject}'])
+
+# %%
 # Explore the dataset contents
 # ----------------------------
 #
 # We can use MNE-BIDS to print a tree of all
 # included files and folders. We pass the ``max_depth`` parameter to
-# `mne_bids.print_dir_tree` to the output to three levels of folders, for
+# `mne_bids.print_dir_tree` to the output to four levels of folders, for
 # better readability in this example.
 
-print_dir_tree(bids_root, max_depth=3)
+print_dir_tree(bids_root, max_depth=4)
 
-###############################################################################
+# %%
 # We can even ask MNE-BIDS to produce a human-readbale summary report
 # on the dataset contents.
 
 print(make_report(bids_root))
 
-###############################################################################
+# %%
 # Now it's time to get ready for reading some of the data! First, we need to
-# create a `mne_bids.BIDSPath`, which is the working horse object of MNE-BIDS
-# when it comes to file and folder operations.
+# create an :class:`mne_bids.BIDSPath`, which is the workhorse object of
+# MNE-BIDS when it comes to file and folder operations.
 #
-# For now, we're interested only in the MEG data in the BIDS root directory
-# of the ``somato`` dataset.
+# For now, we're interested only in the EEG data in the BIDS root directory
+# of the Parkinson's disease patient dataset. There were two sessions, one
+# where the patients took their regular anti-Parkinsonian medications and
+# one where they abstained for more than twelve hours. Let's start with the
+# off-medication session.
 
-datatype = 'meg'
-bids_path = BIDSPath(root=bids_root, datatype=datatype)
+datatype = 'eeg'
+session = 'off'
+bids_path = BIDSPath(root=bids_root, session=session, datatype=datatype)
 
-###############################################################################
+# %%
 # We can now retrieve a list of all MEG-related files in the dataset:
 
 print(bids_path.match())
 
-###############################################################################
+# %%
 # The returned list contains ``BIDSpaths`` of 3 files:
-# ``sub-01_task-somato_channels.tsv``, ``sub-01_task-somato_events.tsv``, and
-# ``sub-01_task-somato_meg.fif``.
+# ``sub-pd6_ses-off_task-rest_channels.tsv``,
+# ``sub-pd6_ses-off_task-rest_events.tsv``, and
+# ``sub-pd6_ses-off_task-rest_eeg.bdf``.
 # The first two are so-called sidecar files that contain information on the
 # recording channels and experimental events, and the third one is the actual
-# MEG data file.
+# data file.
 #
 # Prepare reading the data
 # ------------------------
 #
-# There is obviously only one subject (``01``) and one experimental task
-# (``somato``). Let's use this knowledge to create a new ``BIDSPath`` with
-# all the information required to actually read the MEG data. We also need to
+# There is only one subject and one experimental task (``rest``).
+# Let's use this knowledge to create a new ``BIDSPath`` with
+# all the information required to actually read the EEG data. We also need to
 # pass a ``suffix``, which is the last part of the filename just before the
 # extension -- ``'channels'`` and ``'events'`` for the two TSV files in
-# our example, and ``'meg'`` for MEG raw data. For MEG and EEG raw data, the
-# suffix is identical to the datatype, so don't let yourselve be confused here!
+# our example, and ``'eeg'`` for EEG raw data. For MEG and EEG raw data, the
+# suffix is identical to the datatype, so don't let yourself be confused here!
 
-bids_root = somato.data_path()
-datatype = 'meg'
-subject = '01'
-task = 'somato'
-suffix = 'meg'
+task = 'rest'
+suffix = 'eeg'
 
-bids_path = BIDSPath(subject=subject, task=task, suffix=suffix,
-                     datatype=datatype, root=bids_root)
+bids_path = BIDSPath(subject=subject, session=session, task=task,
+                     suffix=suffix, datatype=datatype, root=bids_root)
 
-###############################################################################
+# %%
 # Now let's print the contents of ``bids_path``.
 
 print(bids_path)
 
-###############################################################################
+# %%
 # You probably noticed two things: Firstly, this looks like an ordinary string
 # now, not like the more-or-less neatly formatted output we saw before. And
 # secondly, that there's suddenly a filename extension which we never specified
@@ -125,16 +149,16 @@ print(bids_path)
 
 bids_path
 
-###############################################################################
+# %%
 # The ``root`` here is – you guessed it – the directory we passed via the
 # ``root`` parameter: the "home" of our BIDS dataset. The ``datatype``, again,
 # is self-explanatory. The ``basename``, on the other hand, is created
 # automatically based on the suffix and **BIDS entities**  we passed to
-# ``BIDSPath``: in our case, ``subject`` and ``task``.
+# ``BIDSPath``: in our case, ``subject``, ``session`` and ``task``.
 #
 # .. note::
 #   There are many more supported entities, the most-commonly used among them
-#   probably being ``session``. Please see
+#   probably being ``acquisition``. Please see
 #   :ref:`our introduction to BIDSPath <bidspath-example>` to learn more
 #   about entities, ``basename``, and ``BIDSPath`` in general.
 #
@@ -143,9 +167,9 @@ bids_path
 # employs some heuristics to auto-detect some missing filename components.
 # Omitting the filename extension in your script can make your code
 # more portable. Note that, however, you **can** explicitly specify an
-# extension too, by passing e.g. ``extension='.fif'`` to ``BIDSPath``.
+# extension too, by passing e.g. ``extension='.bdf'`` to ``BIDSPath``.
 
-###############################################################################
+# %%
 # Read the data
 # -------------
 #
@@ -153,7 +177,7 @@ bids_path
 
 raw = read_raw_bids(bids_path=bids_path, verbose=False)
 
-###############################################################################
+# %%
 # Now we can inspect the ``raw`` object to check that it contains to correct
 # metadata.
 #
@@ -161,28 +185,28 @@ raw = read_raw_bids(bids_path=bids_path, verbose=False)
 
 print(raw.info['subject_info'])
 
-###############################################################################
+# %%
 # Power line frequency is here.
 
 print(raw.info['line_freq'])
 
-###############################################################################
+# %%
 # Sampling frequency is here.
 
 print(raw.info['sfreq'])
 
-###############################################################################
+# %%
 # Events are now Annotations
 print(raw.annotations)
 
-###############################################################################
+# %%
 # Plot the raw data.
 
 raw.plot()
 
-###############################################################################
+# %%
 # .. LINKS
 #
-# .. _mne_somato_data:
-#    https://mne.tools/dev/generated/mne.datasets.somato.data_path.html
+# .. _parkinsons_eeg_dataset:
+#    https://openneuro.org/datasets/ds002778
 #
