@@ -3,7 +3,7 @@ from mne import io
 from mne.io.constants import FIFF
 
 
-BIDS_VERSION = "1.4.0"
+BIDS_VERSION = "1.6.0"
 
 DOI = """https://doi.org/10.21105/joss.01896"""
 
@@ -47,14 +47,14 @@ meg_manufacturers = {
 }
 
 eeg_manufacturers = {'.vhdr': 'BrainProducts', '.eeg': 'BrainProducts',
-                     '.edf': 'n/a', '.bdf': 'Biosemi', '.set': 'n/a',
-                     '.fdt': 'n/a',
+                     '.edf': 'n/a', '.EDF': 'n/a', '.bdf': 'Biosemi',
+                     '.set': 'n/a', '.fdt': 'n/a',
                      '.lay': 'Persyst', '.dat': 'Persyst',
                      '.EEG': 'Nihon Kohden'}
 
 ieeg_manufacturers = {'.vhdr': 'BrainProducts', '.eeg': 'BrainProducts',
-                      '.edf': 'n/a', '.set': 'n/a', '.fdt': 'n/a',
-                      '.mef': 'n/a', '.nwb': 'n/a',
+                      '.edf': 'n/a', '.EDF': 'n/a', '.set': 'n/a',
+                      '.fdt': 'n/a', '.mef': 'n/a', '.nwb': 'n/a',
                       '.lay': 'Persyst', '.dat': 'Persyst',
                       '.EEG': 'Nihon Kohden'}
 
@@ -62,7 +62,8 @@ ieeg_manufacturers = {'.vhdr': 'BrainProducts', '.eeg': 'BrainProducts',
 reader = {'.con': io.read_raw_kit, '.sqd': io.read_raw_kit,
           '.fif': io.read_raw_fif, '.pdf': io.read_raw_bti,
           '.ds': io.read_raw_ctf, '.vhdr': io.read_raw_brainvision,
-          '.edf': io.read_raw_edf, '.bdf': io.read_raw_bdf,
+          '.edf': io.read_raw_edf, '.EDF': io.read_raw_edf,
+          '.bdf': io.read_raw_bdf,
           '.set': io.read_raw_eeglab, '.lay': io.read_raw_persyst,
           '.EEG': io.read_raw_nihon}
 
@@ -85,13 +86,13 @@ IGNORED_CHANNELS = {'KIT/Yokogawa': ['STI 014'],
 
 allowed_extensions_meg = ['.con', '.sqd', '.fif', '.pdf', '.ds']
 allowed_extensions_eeg = ['.vhdr',  # BrainVision, accompanied by .vmrk, .eeg
-                          '.edf',  # European Data Format
+                          '.edf', '.EDF',  # European Data Format
                           '.bdf',  # Biosemi
                           '.set',  # EEGLAB, potentially accompanied by .fdt
                           ]
 
 allowed_extensions_ieeg = ['.vhdr',  # BrainVision, accompanied by .vmrk, .eeg
-                           '.edf',  # European Data Format
+                           '.edf', '.EDF',  # European Data Format
                            '.set',  # EEGLAB, potentially accompanied by .fdt
                            '.mef',  # MEF: Multiscale Electrophysiology File
                            '.nwb',  # Neurodata without borders
@@ -130,7 +131,8 @@ ALLOWED_FILENAME_EXTENSIONS = (
     ALLOWED_INPUT_EXTENSIONS +
     ['.json', '.tsv', '.tsv.gz', '.nii', '.nii.gz'] +
     ['.pos', '.eeg', '.vmrk'] +  # extra datatype-specific metadata files.
-    ['.dat', '.EEG']  # extra eeg extensions
+    ['.dat', '.EEG'] +  # extra eeg extensions
+    ['.mrk']  # KIT/Yokogawa/Ricoh marker coil
 )
 
 # allowed BIDS path entities
@@ -144,6 +146,9 @@ ALLOWED_PATH_ENTITIES_SHORT = {'sub': 'subject', 'ses': 'session',
                                'space': 'space', 'rec': 'recording',
                                'split': 'split', 'suffix': 'suffix'}
 
+# Annotations to never remove during reading or writing
+ANNOTATIONS_TO_KEEP = ('BAD_ACQ_SKIP',)
+
 coordsys_standard_template = [
     'ICBM452AirSpace',
     'ICBM452Warp5Space',
