@@ -1,5 +1,6 @@
 """BIDS compatible path functionality."""
 # Authors: Adam Li <adam2392@gmail.com>
+#          Stefan Appelhoff <stefan.appelhoff@mailbox.org>
 #
 # License: BSD (3-clause)
 import glob
@@ -12,6 +13,8 @@ from copy import deepcopy
 from os import path as op
 from pathlib import Path
 from datetime import datetime
+import json
+from typing import Optional
 
 import numpy as np
 from mne.utils import warn, logger, _validate_type
@@ -26,7 +29,7 @@ from mne_bids.utils import (_check_key_val, _check_empty_room_basename,
                             param_regex, _ensure_tuple)
 
 
-def _get_matched_empty_room(bids_path):
+def _find_matched_empty_room(bids_path):
     """Get matching empty-room file for an MEG recording."""
     # Check whether we have a BIDS root.
     bids_root = bids_path.root
@@ -42,11 +45,7 @@ def _get_matched_empty_room(bids_path):
     bids_fname = bids_path.update(suffix=datatype,
                                   root=bids_root).fpath
     _, ext = _parse_ext(bids_fname)
-    extra_params = None
-    if ext == '.fif':
-        extra_params = dict(allow_maxshield=True)
-
-    raw = read_raw_bids(bids_path=bids_path, extra_params=extra_params)
+    raw = read_raw_bids(bids_path=bids_path)
     if raw.info['meas_date'] is None:
         raise ValueError('The provided recording does not have a measurement '
                          'date set. Cannot get matching empty-room file.')
@@ -165,19 +164,19 @@ class BIDSPath(object):
     subject : str | None
         The subject ID. Corresponds to "sub".
     session : str | None
-        The session for a item. Corresponds to "ses".
+        The acquisition session. Corresponds to "ses".
     task : str | None
-        The task for a item. Corresponds to "task".
+        The experimental task. Corresponds to "task".
     acquisition: str | None
-        The acquisition parameters for the item. Corresponds to "acq".
+        The acquisition parameters. Corresponds to "acq".
     run : int | None
-        The run number for this item. Corresponds to "run".
+        The run number. Corresponds to "run".
     processing : str | None
-        The processing label for this item. Corresponds to "proc".
+        The processing label. Corresponds to "proc".
     recording : str | None
-        The recording name for this item. Corresponds to "rec".
+        The recording name. Corresponds to "rec".
     space : str | None
-        The coordinate space for an anatomical or sensor position
+        The coordinate space for anatomical and sensor location
         files (e.g., ``*_electrodes.tsv``, ``*_markers.mrk``).
         Corresponds to "space".
         Note that valid values for ``space`` must come from a list
@@ -196,12 +195,10 @@ class BIDSPath(object):
     extension : str | None
         The extension of the filename. E.g., ``'.json'``.
     datatype : str
-        The "data type" of folder being created at the end of the folder
-        hierarchy. E.g., ``'anat'``, ``'func'``, ``'eeg'``, ``'meg'``,
-        ``'ieeg'``, etc.
+        The BIDS data type, e.g., ``'anat'``, ``'func'``, ``'eeg'``, ``'meg'``,
+        ``'ieeg'``.
     root : str | pathlib.Path | None
-        The root for the filename to be created. E.g., a path to the folder
-        in which you wish to create a file with this name.
+        The root directory of the BIDS dataset.
     check : bool
         If ``True``, enforces BIDS conformity. Defaults to ``True``.
 
@@ -210,7 +207,8 @@ class BIDSPath(object):
     entities : dict
         The dictionary of the BIDS entities and their values:
         ``subject``, ``session``, ``task``, ``acquisition``,
-        ``run``, ``processing``, ``space``, ``recording`` and ``suffix``.
+        ``run``, ``processing``, ``space``, ``recording``, ``split``,
+        ``suffix``, and ``extension``.
     datatype : str | None
         The data type, i.e., one of ``'meg'``, ``'eeg'``, ``'ieeg'``,
         ``'anat'``.
@@ -227,37 +225,50 @@ class BIDSPath(object):
 
     Examples
     --------
+    Generate a BIDSPath object and inspect it
+
     >>> bids_path = BIDSPath(subject='test', session='two', task='mytask',
-                             suffix='ieeg', extension='.edf')
+    ...                      suffix='ieeg', extension='.edf')
     >>> print(bids_path.basename)
     sub-test_ses-two_task-mytask_ieeg.edf
     >>> bids_path
-    BIDSPath(root: None,
+    BIDSPath(
+    root: None
+    datatype: ieeg
     basename: sub-test_ses-two_task-mytask_ieeg.edf)
-    >>> # copy and update multiple entities at once
+
+    Copy and update multiple entities at once
+
     >>> new_bids_path = bids_path.copy().update(subject='test2',
-                                                session='one')
+    ...                                         session='one')
     >>> print(new_bids_path.basename)
     sub-test2_ses-one_task-mytask_ieeg.edf
-    >>> # printing the BIDSPath will show relative path when
-    >>> # root is not set
+
+    Printing a BIDSPath will show a relative path when `root` is not set
+
     >>> print(new_bids_path)
     sub-test2/ses-one/ieeg/sub-test2_ses-one_task-mytask_ieeg.edf
-    >>> new_bids_path.update(suffix='channels', extension='.tsv')
-    >>> # setting suffix without an identifiable datatype will
-    >>> # result in a wildcard at the datatype directory level
+
+    Setting `suffix` without an identifiable datatype will make
+    BIDSPath try to guess the datatype
+
+    >>> new_bids_path = new_bids_path.update(suffix='channels',
+    ...                                      extension='.tsv')
     >>> print(new_bids_path)
-    sub-test2/ses-one/*/sub-test2_ses-one_task-mytask_channels.tsv
-    >>> # set a root for the BIDS dataset
-    >>> new_bids_path.update(root='/bids_dataset')
-    >>> print(new_bids_path.root)
+    sub-test2/ses-one/ieeg/sub-test2_ses-one_task-mytask_channels.tsv
+
+    You can set a new root for the BIDS dataset. Let's see what the
+    different properties look like for our object:
+
+    >>> new_bids_path = new_bids_path.update(root='/bids_dataset')
+    >>> print(new_bids_path.root.as_posix())
     /bids_dataset
     >>> print(new_bids_path.basename)
-    sub-test2_ses-one_task-mytask_ieeg.edf
+    sub-test2_ses-one_task-mytask_channels.tsv
     >>> print(new_bids_path)
-    /bids_dataset/sub-test2/ses-one/ieeg/sub-test2_ses-one_task-mytask_ieeg.edf
-    >>> print(new_bids_path.directory)
-    /bids_dataset/sub-test2/ses-one/ieeg/
+    /bids_dataset/sub-test2/ses-one/ieeg/sub-test2_ses-one_task-mytask_channels.tsv
+    >>> print(new_bids_path.directory.as_posix())
+    /bids_dataset/sub-test2/ses-one/ieeg
 
     Notes
     -----
@@ -361,14 +372,133 @@ class BIDSPath(object):
             data_path = op.join(data_path, self.datatype)
         return Path(data_path)
 
+    @property
+    def subject(self) -> Optional[str]:
+        """The subject ID."""
+        return self._subject
+
+    @subject.setter
+    def subject(self, value):
+        self.update(subject=value)
+
+    @property
+    def session(self) -> Optional[str]:
+        """The acquisition session."""
+        return self._session
+
+    @session.setter
+    def session(self, value):
+        self.update(session=value)
+
+    @property
+    def task(self) -> Optional[str]:
+        """The experimental task."""
+        return self._task
+
+    @task.setter
+    def task(self, value):
+        self.update(task=value)
+
+    @property
+    def run(self) -> Optional[str]:
+        """The run number."""
+        return self._run
+
+    @run.setter
+    def run(self, value):
+        self.update(run=value)
+
+    @property
+    def acquisition(self) -> Optional[str]:
+        """The acquisition parameters."""
+        return self._acquisition
+
+    @acquisition.setter
+    def acquisition(self, value):
+        self.update(acquisition=value)
+
+    @property
+    def processing(self) -> Optional[str]:
+        """The processing label."""
+        return self._processing
+
+    @processing.setter
+    def processing(self, value):
+        self.update(processing=value)
+
+    @property
+    def recording(self) -> Optional[str]:
+        """The recording name."""
+        return self._recording
+
+    @recording.setter
+    def recording(self, value):
+        self.update(recording=value)
+
+    @property
+    def space(self) -> Optional[str]:
+        """The coordinate space for an anatomical or sensor position file."""
+        return self._space
+
+    @space.setter
+    def space(self, value):
+        self.update(space=value)
+
+    @property
+    def suffix(self) -> Optional[str]:
+        """The filename suffix."""
+        return self._suffix
+
+    @suffix.setter
+    def suffix(self, value):
+        self.update(suffix=value)
+
+    @property
+    def root(self) -> Optional[Path]:
+        """The root directory of the BIDS dataset."""
+        return self._root
+
+    @root.setter
+    def root(self, value):
+        self.update(root=value)
+
+    @property
+    def datatype(self) -> Optional[str]:
+        """The BIDS data type, e.g. ``'anat'``, ``'meg'``, ``'eeg'``."""
+        return self._datatype
+
+    @datatype.setter
+    def datatype(self, value):
+        self.update(datatype=value)
+
+    @property
+    def split(self) -> Optional[str]:
+        """The split of the continuous recording file for ``.fif`` data."""
+        return self._split
+
+    @split.setter
+    def split(self, value):
+        self.update(split=value)
+
+    @property
+    def extension(self) -> Optional[str]:
+        """The extension of the filename, including a leading period."""
+        return self._extension
+
+    @extension.setter
+    def extension(self, value):
+        self.update(extension=value)
+
     def __str__(self):
         """Return the string representation of the path."""
-        return str(self.fpath)
+        return str(self.fpath.as_posix())
 
     def __repr__(self):
         """Representation in the style of `pathlib.Path`."""
+        root = self.root.as_posix() if self.root is not None else None
+
         return f'{self.__class__.__name__}(\n' \
-               f'root: {self.root}\n' \
+               f'root: {root}\n' \
                f'datatype: {self.datatype}\n' \
                f'basename: {self.basename})'
 
@@ -523,7 +653,7 @@ class BIDSPath(object):
         **kwargs : dict
             It can contain updates for valid BIDS path entities:
             'subject', 'session', 'task', 'acquisition', 'processing', 'run',
-            'recording', 'space', 'suffix'
+            'recording', 'space', 'suffix', 'split', 'extension',
             or updates for 'root' or 'datatype'.
 
         Returns
@@ -537,13 +667,13 @@ class BIDSPath(object):
         :func:`mne_bids.BIDSPath`:
 
         >>> bids_path = BIDSPath(subject='test', session='two',
-                                     task='mytask', suffix='channels',
-                                     extension='.tsv')
+        ...                      task='mytask', suffix='channels',
+        ...                      extension='.tsv')
         >>> print(bids_path.basename)
         sub-test_ses-two_task-mytask_channels.tsv
         >>> # Then, one can update this `BIDSPath` object in place
-        >>> bids_path.update(acquisition='test', suffix='ieeg',
-                             extension='.vhdr', task=None)
+        >>> bids_path = bids_path.update(acquisition='test', suffix='ieeg',
+        ...                              extension='.vhdr', task=None)
         >>> print(bids_path.basename)
         sub-test_ses-two_acq-test_ieeg.vhdr
         """
@@ -601,12 +731,12 @@ class BIDSPath(object):
             # set entity value, ensuring `root` is a Path
             if val is not None and key == 'root':
                 val = Path(val).expanduser()
-            setattr(self, key, val)
+            setattr(self, f'_{key}', val)
 
         # infer datatype if suffix is uniquely the datatype
         if self.datatype is None and \
                 self.suffix in SUFFIX_TO_DATATYPE:
-            self.datatype = SUFFIX_TO_DATATYPE[self.suffix]
+            self._datatype = SUFFIX_TO_DATATYPE[self.suffix]
 
         # Perform a check of the entities.
         self._check()
@@ -746,14 +876,49 @@ class BIDSPath(object):
         This will only work if the ``.root`` attribute of the
         :class:`mne_bids.BIDSPath` instance has been set.
 
+        .. note:: If the sidecar JSON file contains an ``AssociatedEmptyRoom``
+                  entry, the empty-room recording specified there will be used.
+                  Otherwise, this method will try to find the best-matching
+                  empty-room recording based on measurement date.
+
         Returns
         -------
         BIDSPath | None
             The path corresponding to the best-matching empty-room measurement.
-            Returns None if none was found.
-
+            Returns ``None`` if none was found.
         """
-        return _get_matched_empty_room(self)
+        if self.datatype not in ('meg', None):
+            raise ValueError('Empty-room data is only supported for MEG '
+                             'datasets')
+
+        if self.root is None:
+            raise ValueError('The root of the "bids_path" must be set. '
+                             'Please use `bids_path.update(root="<root>")` '
+                             'to set the root of the BIDS folder to read.')
+
+        sidecar_fname = _find_matching_sidecar(self, extension='.json')
+        with open(sidecar_fname, 'r', encoding='utf-8') as f:
+            sidecar_json = json.load(f)
+
+        if 'AssociatedEmptyRoom' in sidecar_json:
+            logger.info('Using "AssociatedEmptyRoom" entry from MEG sidecar '
+                        'file to retrieve empty-room path.')
+            emptytoom_path = sidecar_json['AssociatedEmptyRoom']
+            emptyroom_entities = get_entities_from_fname(emptytoom_path)
+            er_bids_path = BIDSPath(root=self.root, datatype='meg',
+                                    **emptyroom_entities)
+        else:
+            logger.info(
+                'The MEG sidecar file does not contain an '
+                '"AssociatedEmptyRoom" entry. Will try to find a matching '
+                'empty-room recording based on the measurement date …'
+            )
+            er_bids_path = _find_matched_empty_room(self)
+
+        if er_bids_path is not None:
+            assert er_bids_path.fpath.exists()
+
+        return er_bids_path
 
     @property
     def meg_calibration_fpath(self):
@@ -998,16 +1163,16 @@ def get_entities_from_fname(fname, on_error='raise'):
     --------
     >>> fname = 'sub-01_ses-exp_run-02_meg.fif'
     >>> get_entities_from_fname(fname)
-    {'subject': '01',
-    'session': 'exp',
-    'task': None,
-    'acquisition': None,
-    'run': '02',
-    'processing': None,
-    'space': None,
-    'recording': None,
-    'split': None,
-    'suffix': 'meg'}
+    {'subject': '01', \
+'session': 'exp', \
+'task': None, \
+'acquisition': None, \
+'run': '02', \
+'processing': None, \
+'space': None, \
+'recording': None, \
+'split': None, \
+'suffix': 'meg'}
     """
     if on_error not in ('warn', 'raise', 'ignore'):
         raise ValueError(f'Acceptable values for on_error are: warn, raise, '
@@ -1197,7 +1362,20 @@ def get_entity_vals(root, entity_key, *, ignore_subjects='emptyroom',
     Parameters
     ----------
     root : str | pathlib.Path
-        Path to the root of the BIDS directory.
+        Path to the "root" directory from which to start traversing to gather
+        BIDS entities from file- and folder names. This will commonly be the
+        BIDS root, but it may also be a subdirectory inside of a BIDS dataset,
+        e.g., the ``sub-X`` directory of a hypothetical subject ``X``.
+
+        .. note:: This function searches the names of all files and directories
+                  nested within ``root``. Depending on the size of your
+                  dataset and storage system, searching the entire BIDS dataset
+                  may take a **considerable** amount of time (seconds up to
+                  several minutes). If you find yourself running into such
+                  performance issues, consider limiting the search to only a
+                  subdirectory in the dataset, e.g., to a single subject or
+                  session only.
+
     entity_key : str
         The name of the entity key to search for.
     ignore_subjects : str | iterable | None
@@ -1236,12 +1414,12 @@ def get_entity_vals(root, entity_key, *, ignore_subjects='emptyroom',
 
     Examples
     --------
-    >>> root = os.path.expanduser('~/mne_data/eeg_matchingpennies')
-    >>> entity_key = 'sub'
+    >>> root = Path('./mne_bids/tests/data/tiny_bids').absolute()
+    >>> entity_key = 'subject'
     >>> get_entity_vals(root, entity_key)
-    ['05', '06', '07', '08', '09', '10', '11']
+    ['01']
     >>> get_entity_vals(root, entity_key, with_key=True)
-    ['sub-05', 'sub-06', 'sub-07', 'sub-08', 'sub-09', 'sub-10', 'sub-11']
+    ['sub-01']
 
     Notes
     -----
