@@ -21,8 +21,8 @@ with warnings.catch_warnings():
                             message="can't resolve package",
                             category=ImportWarning)
     import mne
-
-from mne.utils import requires_nibabel, object_diff
+from mne.io.constants import FIFF
+from mne.utils import requires_nibabel, object_diff, requires_version
 from mne.utils import assert_dig_allclose
 from mne.datasets import testing, somato
 
@@ -35,7 +35,7 @@ from mne_bids.tsv_handler import _to_tsv, _from_tsv
 from mne_bids.utils import (_write_json)
 from mne_bids.sidecar_updates import _update_sidecar
 from mne_bids.path import _find_matching_sidecar
-from mne_bids.write import write_anat, write_raw_bids
+from mne_bids.write import write_anat, write_raw_bids, get_anat_landmarks
 
 subject_id = '01'
 session_id = '01'
@@ -59,11 +59,15 @@ somato_path = somato.data_path()
 somato_raw_fname = op.join(somato_path, 'sub-01', 'meg',
                            'sub-01_task-somato_meg.fif')
 
+# Data with cHPI info
+raw_fname_chpi = op.join(data_path, 'SSS', 'test_move_anon_raw.fif')
+
 warning_str = dict(
     channel_unit_changed='ignore:The unit for chann*.:RuntimeWarning:mne',
     meas_date_set_to_none="ignore:.*'meas_date' set to None:RuntimeWarning:"
                           "mne",
     nasion_not_found='ignore:.*nasion not found:RuntimeWarning:mne',
+    maxshield='ignore:.*Internal Active Shielding:RuntimeWarning:mne'
 )
 
 
@@ -203,6 +207,7 @@ def test_get_head_mri_trans(tmpdir):
                 'Visual/Right': 4, 'Smiley': 5, 'Button': 32}
     events_fname = op.join(data_path, 'MEG', 'sample',
                            'sample_audvis_trunc_raw-eve.fif')
+    subjects_dir = op.join(data_path, 'subjects')
 
     # Drop unknown events.
     events = mne.read_events(events_fname)
@@ -214,9 +219,11 @@ def test_get_head_mri_trans(tmpdir):
     write_raw_bids(raw, bids_path, events_data=events, event_id=event_id,
                    overwrite=False)
 
-    # We cannot recover trans, if no MRI has yet been written
-    with pytest.raises(RuntimeError):
-        estimated_trans = get_head_mri_trans(bids_path=bids_path)
+    # We cannot recover trans if no MRI has yet been written
+    with pytest.raises(RuntimeError, match='Did not find any T1w'):
+        estimated_trans = get_head_mri_trans(
+            bids_path=bids_path, fs_subject='sample',
+            fs_subjects_dir=subjects_dir)
 
     # Write some MRI data and supply a `trans` so that a sidecar gets written
     trans = mne.read_trans(raw_fname.replace('_raw.fif', '-trans.fif'))
@@ -226,28 +233,92 @@ def test_get_head_mri_trans(tmpdir):
     t1w_mgh = op.join(data_path, 'subjects', 'sample', 'mri', 'T1.mgz')
     t1w_mgh = nib.load(t1w_mgh)
 
-    t1w_bidspath = BIDSPath(subject=subject_id, session=session_id,
-                            acquisition=acq, root=tmpdir)
-    t1w_bidspath = write_anat(t1w_mgh, bids_path=t1w_bidspath,
-                              raw=raw, trans=trans, verbose=True)
-    anat_dir = t1w_bidspath.directory
+    landmarks = get_anat_landmarks(
+        t1w_mgh, raw.info, trans, fs_subject='sample',
+        fs_subjects_dir=subjects_dir)
+    t1w_bids_path = write_anat(
+        t1w_mgh, bids_path=bids_path, landmarks=landmarks, verbose=True)
+    anat_dir = bids_path.directory
 
     # Try to get trans back through fitting points
-    estimated_trans = get_head_mri_trans(bids_path=bids_path)
+    estimated_trans = get_head_mri_trans(
+        bids_path=bids_path, fs_subject='sample', fs_subjects_dir=subjects_dir)
 
     assert trans['from'] == estimated_trans['from']
     assert trans['to'] == estimated_trans['to']
     assert_almost_equal(trans['trans'], estimated_trans['trans'])
-    print(trans)
-    print(estimated_trans)
 
     # provoke an error by introducing NaNs into MEG coords
+    raw.info['dig'][0]['r'] = np.full(3, np.nan)
+    sh.rmtree(anat_dir)
+    bad_landmarks = get_anat_landmarks(t1w_mgh, raw.info, trans, 'sample',
+                                       op.join(data_path, 'subjects'))
+    write_anat(t1w_mgh, bids_path=t1w_bids_path, landmarks=bad_landmarks)
     with pytest.raises(RuntimeError, match='AnatomicalLandmarkCoordinates'):
-        raw.info['dig'][0]['r'] = np.ones(3) * np.nan
-        sh.rmtree(anat_dir)
-        bids_path = write_anat(t1w_mgh, bids_path=t1w_bidspath,
-                               raw=raw, trans=trans, verbose=True)
-        estimated_trans = get_head_mri_trans(bids_path=bids_path)
+        estimated_trans = get_head_mri_trans(bids_path=t1w_bids_path,
+                                             fs_subject='sample',
+                                             fs_subjects_dir=subjects_dir)
+
+    # test we are permissive for different casings of landmark names in the
+    # sidecar, and also accept "nasion" instead of just "NAS"
+    raw = _read_raw_fif(raw_fname)
+    write_raw_bids(raw, bids_path, events_data=events, event_id=event_id,
+                   overwrite=True)  # overwrite with new acq
+    t1w_bids_path = write_anat(t1w_mgh, bids_path=bids_path,
+                               landmarks=landmarks, overwrite=True)
+
+    t1w_json_fpath = t1w_bids_path.copy().update(extension='.json').fpath
+    with t1w_json_fpath.open('r', encoding='utf-8') as f:
+        t1w_json = json.load(f)
+
+    coords = t1w_json['AnatomicalLandmarkCoordinates']
+    coords['lpa'] = coords['LPA']
+    coords['Rpa'] = coords['RPA']
+    coords['Nasion'] = coords['NAS']
+    del coords['LPA'], coords['RPA'], coords['NAS']
+
+    _write_json(t1w_json_fpath, t1w_json, overwrite=True)
+
+    estimated_trans = get_head_mri_trans(
+        bids_path=bids_path,
+        fs_subject='sample', fs_subjects_dir=subjects_dir)
+    assert_almost_equal(trans['trans'], estimated_trans['trans'])
+
+    # Test t1_bids_path parameter
+    #
+    # Case 1: different BIDS roots
+    meg_bids_path = _bids_path.copy().update(root=tmpdir / 'meg_root')
+    t1_bids_path = _bids_path.copy().update(root=tmpdir / 'mri_root')
+    raw = _read_raw_fif(raw_fname)
+
+    write_raw_bids(raw, bids_path=meg_bids_path)
+    landmarks = get_anat_landmarks(
+        t1w_mgh, raw.info, trans, fs_subject='sample',
+        fs_subjects_dir=subjects_dir)
+    write_anat(t1w_mgh, bids_path=t1_bids_path, landmarks=landmarks)
+    read_trans = get_head_mri_trans(
+        bids_path=meg_bids_path, t1_bids_path=t1_bids_path,
+        fs_subject='sample', fs_subjects_dir=subjects_dir)
+    assert np.allclose(trans['trans'], read_trans['trans'])
+
+    # Case 2: different sessions
+    raw = _read_raw_fif(raw_fname)
+    meg_bids_path = _bids_path.copy().update(root=tmpdir / 'session_test',
+                                             session='01')
+    t1_bids_path = meg_bids_path.copy().update(session='02')
+
+    write_raw_bids(raw, bids_path=meg_bids_path)
+    write_anat(t1w_mgh, bids_path=t1_bids_path, landmarks=landmarks)
+    read_trans = get_head_mri_trans(
+        bids_path=meg_bids_path, t1_bids_path=t1_bids_path,
+        fs_subject='sample', fs_subjects_dir=subjects_dir)
+    assert np.allclose(trans['trans'], read_trans['trans'])
+
+    # Test that incorrect subject directory throws error
+    with pytest.raises(ValueError, match='Could not find'):
+        estimated_trans = get_head_mri_trans(
+            bids_path=bids_path, fs_subject='bad',
+            fs_subjects_dir=subjects_dir)
 
 
 def test_handle_events_reading(tmpdir):
@@ -307,6 +378,28 @@ def test_handle_events_reading(tmpdir):
     assert ids == ['n/a']
 
 
+@pytest.mark.filterwarnings(warning_str['channel_unit_changed'])
+def test_keep_essential_annotations(tmpdir):
+    """Test that essential Annotations are not omitted during I/O roundtrip."""
+    raw = _read_raw_fif(raw_fname)
+    annotations = mne.Annotations(onset=[raw.times[0]], duration=[1],
+                                  description=['BAD_ACQ_SKIP'])
+    raw.set_annotations(annotations)
+
+    # Write data, remove events.tsv, then try to read again
+    bids_path = BIDSPath(subject='01', task='task', datatype='meg',
+                         root=tmpdir)
+    with pytest.warns(RuntimeWarning, match='Acquisition skips detected'):
+        write_raw_bids(raw, bids_path, overwrite=True)
+
+    bids_path.copy().update(suffix='events', extension='.tsv').fpath.unlink()
+    raw_read = read_raw_bids(bids_path)
+
+    assert len(raw_read.annotations) == len(raw.annotations) == 1
+    assert (raw_read.annotations[0]['description'] ==
+            raw.annotations[0]['description'])
+
+
 @pytest.mark.filterwarnings(warning_str['channel_unit_changed'])
 def test_handle_scans_reading(tmpdir):
     """Test reading data from a BIDS scans.tsv file."""
@@ -350,7 +443,7 @@ def test_handle_scans_reading(tmpdir):
 
 @pytest.mark.filterwarnings(warning_str['channel_unit_changed'])
 def test_handle_info_reading(tmpdir):
-    """Test reading information from a BIDS sidecar.json file."""
+    """Test reading information from a BIDS sidecar JSON file."""
     # read in USA dataset, so it should find 50 Hz
     raw = _read_raw_fif(raw_fname)
 
@@ -388,6 +481,30 @@ def test_handle_info_reading(tmpdir):
     with pytest.raises(ValueError, match="PowerLineFrequency .* required"):
         write_raw_bids(raw, bids_path, overwrite=True)
 
+    # check whether there are "Extra points" in raw.info['dig'] if
+    # DigitizedHeadPoints is set to True and not otherwise
+    n_dig_points = 0
+    for dig_point in raw.info['dig']:
+        if dig_point['kind'] == FIFF.FIFFV_POINT_EXTRA:
+            n_dig_points += 1
+    if sidecar_json['DigitizedHeadPoints']:
+        assert n_dig_points > 0
+    else:
+        assert n_dig_points == 0
+
+    # check whether any of NAS/LPA/RPA are present in raw.info['dig']
+    # DigitizedLandmark is set to True, and False otherwise
+    landmark_present = False
+    for dig_point in raw.info['dig']:
+        if dig_point['kind'] in [FIFF.FIFFV_POINT_LPA, FIFF.FIFFV_POINT_RPA,
+                                 FIFF.FIFFV_POINT_NASION]:
+            landmark_present = True
+            break
+    if landmark_present:
+        assert sidecar_json['DigitizedLandmarks'] is True
+    else:
+        assert sidecar_json['DigitizedLandmarks'] is False
+
     # make a copy of the sidecar in "derivatives/"
     # to check that we make sure we always get the right sidecar
     # in addition, it should not break the sidecar reading
@@ -410,6 +527,44 @@ def test_handle_info_reading(tmpdir):
         assert raw.info['line_freq'] == 55
 
 
+@requires_version('mne', '0.24.dev0')
+@pytest.mark.filterwarnings(warning_str['channel_unit_changed'])
+@pytest.mark.filterwarnings(warning_str['maxshield'])
+def test_handle_chpi_reading(tmpdir):
+    """Test reading of cHPI information."""
+    raw = _read_raw_fif(raw_fname_chpi, allow_maxshield=True)
+    root = tmpdir.mkdir('chpi')
+    bids_path = BIDSPath(subject='01', session='01',
+                         task='audiovisual', run='01',
+                         root=root, datatype='meg')
+    bids_path = write_raw_bids(raw, bids_path)
+
+    raw_read = read_raw_bids(bids_path)
+    assert raw_read.info['hpi_subsystem'] is not None
+
+    # cause conflicts between cHPI info in sidecar and raw data
+    meg_json_path = bids_path.copy().update(suffix='meg', extension='.json')
+    with open(meg_json_path, 'r', encoding='utf-8') as f:
+        meg_json_data = json.load(f)
+
+    # cHPI frequency mismatch
+    meg_json_data_freq_mismatch = meg_json_data.copy()
+    meg_json_data_freq_mismatch['HeadCoilFrequency'][0] = 123
+    _write_json(meg_json_path, meg_json_data_freq_mismatch, overwrite=True)
+
+    with pytest.raises(ValueError, match='cHPI coil frequencies'):
+        raw_read = read_raw_bids(bids_path)
+
+    # cHPI "off" according to sidecar, but present in the data
+    meg_json_data_chpi_mismatch = meg_json_data.copy()
+    meg_json_data_chpi_mismatch['ContinuousHeadLocalization'] = False
+    _write_json(meg_json_path, meg_json_data_chpi_mismatch, overwrite=True)
+
+    raw_read = read_raw_bids(bids_path)
+    assert raw_read.info['hpi_subsystem'] is None
+    assert raw_read.info['hpi_meas'] == []
+
+
 @pytest.mark.filterwarnings(warning_str['nasion_not_found'])
 @pytest.mark.filterwarnings(warning_str['channel_unit_changed'])
 def test_handle_eeg_coords_reading(tmpdir):
@@ -682,12 +837,15 @@ def test_get_head_mri_trans_ctf(fname, tmpdir):
 
     t1w_bids_path = BIDSPath(subject=subject_id, session=session_id,
                              acquisition=acq, root=tmpdir)
-    write_anat(t1w_mgh, bids_path=t1w_bids_path,
-               raw=raw_ctf, trans=trans)
+    landmarks = get_anat_landmarks(
+        t1w_mgh, raw_ctf.info, trans, fs_subject='sample',
+        fs_subjects_dir=op.join(data_path, 'subjects'))
+    write_anat(t1w_mgh, bids_path=t1w_bids_path, landmarks=landmarks)
 
     # Try to get trans back through fitting points
-    estimated_trans = get_head_mri_trans(bids_path=bids_path,
-                                         extra_params=dict(clean_names=True))
+    estimated_trans = get_head_mri_trans(
+        bids_path=bids_path, extra_params=dict(clean_names=True),
+        fs_subject='sample', fs_subjects_dir=op.join(data_path, 'subjects'))
 
     assert_almost_equal(trans['trans'], estimated_trans['trans'])
 
@@ -836,3 +994,60 @@ def test_ignore_exclude_param(tmpdir):
     raw = read_raw_bids(bids_path=bids_path, verbose=False,
                         extra_params=dict(exclude=[ch_name]))
     assert ch_name in raw.ch_names
+
+
+@pytest.mark.filterwarnings(warning_str['channel_unit_changed'])
+def test_channels_tsv_raw_mismatch(tmpdir):
+    """Test behavior when channels.tsv contains channels not found in raw."""
+    bids_path = _bids_path.copy().update(root=tmpdir, datatype='meg',
+                                         task='rest')
+
+    # Remove one channel from the raw data without updating channels.tsv
+    raw = _read_raw_fif(raw_fname, verbose=False)
+    write_raw_bids(raw, bids_path=bids_path, overwrite=True, verbose=False)
+
+    raw_path = bids_path.copy().update(extension='.fif').fpath
+    raw = _read_raw(raw_path, preload=True)
+    raw.drop_channels(ch_names=raw.ch_names[-1])
+    raw.load_data()
+    raw.save(raw_path, overwrite=True)
+
+    with pytest.warns(
+        RuntimeWarning,
+        match='number of channels in the channels.tsv sidecar .* '
+              'does not match the number of channels in the raw data'
+    ):
+        read_raw_bids(bids_path)
+
+    # Remame a channel in the raw data without updating channels.tsv
+    # (number of channels in channels.tsv and raw remains different)
+    ch_name_orig = raw.ch_names[-1]
+    ch_name_new = 'MEGtest'
+    raw.rename_channels({ch_name_orig: ch_name_new})
+    raw.save(raw_path, overwrite=True)
+
+    with pytest.warns(
+        RuntimeWarning,
+        match=f'Cannot set channel type for the following channels, as they '
+              f'are missing in the raw data: {ch_name_orig}'
+    ):
+        read_raw_bids(bids_path)
+
+    # Mark channel as bad in channels.tsv and remove it from the raw data
+    raw = _read_raw_fif(raw_fname, verbose=False)
+    ch_name_orig = raw.ch_names[-1]
+    ch_name_new = 'MEGtest'
+
+    raw.info['bads'] = [ch_name_orig]
+    write_raw_bids(raw, bids_path=bids_path, overwrite=True, verbose=False)
+
+    raw.drop_channels(raw.ch_names[-2])
+    raw.rename_channels({ch_name_orig: ch_name_new})
+    raw.save(raw_path, overwrite=True)
+
+    with pytest.warns(
+        RuntimeWarning,
+        match=f'Cannot set "bad" status for the following channels, as '
+              f'they are missing in the raw data: {ch_name_orig}'
+    ):
+        read_raw_bids(bids_path)
